@@ -20,20 +20,23 @@
 
 
 #include "dpms.h"
-#include "abstractdpmshelper.h"
-#include "xcbdpmshelper.h"
-#include "waylanddpmshelper.h"
+#include <KScreenDpms/Dpms>
 
 #include <powerdevilbackendinterface.h>
 #include <powerdevilcore.h>
 #include <powerdevil_debug.h>
+#include <kwinkscreenhelpereffect.h>
 
 #include <QAction>
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusPendingCall>
 #include <QGuiApplication>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QX11Info>
+#else
+#include <private/qtx11extras_p.h>
+#endif
 #include <QTimer>
 #include <QDebug>
 
@@ -43,26 +46,25 @@
 #include <KLocalizedString>
 #include <KPluginFactory>
 #include <KSharedConfig>
-#include <TabletModeWatcher>
+#include <Kirigami/TabletModeWatcher>
+
+K_PLUGIN_CLASS_WITH_JSON(PowerDevil::BundledActions::DPMS, "powerdevildpmsaction.json")
 
 namespace PowerDevil {
 namespace BundledActions {
-
-DPMS::DPMS(QObject* parent)
+DPMS::DPMS(QObject *parent, const QVariantList &)
     : Action(parent)
-    , m_helper()
+    , m_dpms(new KScreen::Dpms)
 {
     setRequiredPolicies(PowerDevil::PolicyAgent::ChangeScreenSettings);
 
+    // On Wayland, KWin takes care of performing the effect properly
     if (QX11Info::isPlatformX11()) {
-        m_helper.reset(new XcbDpmsHelper);
-    } else if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive)) {
-        m_helper.reset(new WaylandDpmsHelper);
+        auto fadeEffect = new PowerDevil::KWinKScreenHelperEffect(this);
+        connect(this, &DPMS::startFade, fadeEffect, &PowerDevil::KWinKScreenHelperEffect::start);
+        connect(this, &DPMS::stopFade, fadeEffect, &PowerDevil::KWinKScreenHelperEffect::stop);
     }
 
-    // Pretend we're unloading profiles here, as if the action is not enabled, DPMS should be switched off.
-    onProfileUnload();
-
     // Listen to the policy agent
     connect(PowerDevil::PolicyAgent::instance(), &PowerDevil::PolicyAgent::unavailablePoliciesChanged,
             this, &DPMS::onUnavailablePoliciesChanged);
@@ -76,12 +78,10 @@ DPMS::DPMS(QObject* parent)
     QAction *globalAction = actionCollection->addAction(QLatin1String("Turn Off Screen"));
     globalAction->setText(i18nc("@action:inmenu Global shortcut", "Turn Off Screen"));
     connect(globalAction, &QAction::triggered, this, [this] {
-        if (m_helper) {
-            if (m_lockBeforeTurnOff) {
-                lockScreen();
-            }
-            m_helper->trigger(QStringLiteral("TurnOff"));
+        if (m_lockBeforeTurnOff) {
+            lockScreen();
         }
+        m_dpms->switchMode(KScreen::Dpms::Off);
     });
 
     auto powerButtonMode = [globalAction] (bool isTablet) {
@@ -100,21 +100,13 @@ DPMS::~DPMS() = default;
 
 bool DPMS::isSupported()
 {
-    return !m_helper.isNull() && m_helper->isSupported();
-}
-
-void DPMS::onProfileUnload()
-{
-    if (!isSupported()) {
-        return;
-    }
-    m_helper->profileUnloaded();
+    return m_dpms->isSupported();
 }
 
 void DPMS::onWakeupFromIdle()
 {
     if (isSupported()) {
-        m_helper->stopFade();
+        Q_EMIT stopFade();
     }
     if (m_oldKeyboardBrightness > 0) {
         setKeyboardBrightnessHelper(m_oldKeyboardBrightness);
@@ -131,7 +123,7 @@ void DPMS::onIdleTimeout(int msec)
 
     if (msec == m_idleTime * 1000 - 5000) { // fade out screen
         if (isSupported()) {
-            m_helper->startFade();
+            Q_EMIT startFade();
         }
     } else if (msec == m_idleTime * 1000) {
         const int brightness = backend()->brightness(PowerDevil::BackendInterface::Keyboard);
@@ -140,7 +132,7 @@ void DPMS::onIdleTimeout(int msec)
             setKeyboardBrightnessHelper(0);
         }
         if (isSupported()) {
-            m_helper->dpmsTimeout();
+            m_dpms->switchMode(KScreen::Dpms::Off);
         }
     }
 }
@@ -152,14 +144,6 @@ void DPMS::setKeyboardBrightnessHelper(int brightness)
     });
 }
 
-void DPMS::onProfileLoad()
-{
-    if (!isSupported()) {
-        return;
-    }
-    m_helper->profileLoaded();
-}
-
 void DPMS::triggerImpl(const QVariantMap& args)
 {
     QString KEYBOARD_BRIGHTNESS = QStringLiteral("KeyboardBrightness");
@@ -175,7 +159,19 @@ void DPMS::triggerImpl(const QVariantMap& args)
     if (m_lockBeforeTurnOff && (type == "TurnOff" || type == "ToggleOnOff")) {
         lockScreen();
     }
-    m_helper->trigger(args.value(QStringLiteral("Type")).toString());
+    KScreen::Dpms::Mode level = KScreen::Dpms::Mode::On;
+    if (type == QLatin1String("ToggleOnOff")) {
+        level = KScreen::Dpms::Toggle;
+    } else if (type == QLatin1String("TurnOff")) {
+        level = KScreen::Dpms::Off;
+    } else if (type == QLatin1String("Standby")) {
+        level = KScreen::Dpms::Standby;
+    } else if (type == QLatin1String("Suspend")) {
+        level = KScreen::Dpms::Suspend;
+    } else {
+        level = KScreen::Dpms::On;
+    }
+    m_dpms->switchMode(level);
 }
 
 bool DPMS::loadAction(const KConfigGroup& config)
@@ -198,23 +194,7 @@ bool DPMS::onUnloadAction()
 
 void DPMS::onUnavailablePoliciesChanged(PowerDevil::PolicyAgent::RequiredPolicies policies)
 {
-    // only take action if screen inhibit changed
-    PowerDevil::PolicyAgent::RequiredPolicies oldPolicy = m_inhibitScreen;
     m_inhibitScreen = policies & PowerDevil::PolicyAgent::ChangeScreenSettings;
-    if (oldPolicy == m_inhibitScreen) {
-        return;
-    }
-
-    if (m_inhibitScreen) {
-        // Inhibition triggered: disable DPMS
-        if (isSupported()) {
-            m_helper->inhibited();
-        }
-    } else {
-        // Inhibition removed: let's start again
-        onProfileLoad();
-        qCDebug(POWERDEVIL) << "Restoring DPMS features after inhibition release";
-    }
 }
 
 static std::chrono::milliseconds dimAnimationTime()
@@ -236,3 +216,4 @@ void DPMS::lockScreen()
 
 }
 }
+#include "dpms.moc"
